------------------------ 
--
-- Broken Loot
--
-- Usage:
--
-- /bloot
--      display the UI (WORK IN PROGRESS)
-- /bloot broadcast
--      broadcasts all the loot to the raid

local function handler(msg)
  local command, rest = msg:match("^(%S*)%s*(.-)$");
  -- Any leading non-whitespace is captured into command;
  -- the rest (minus leading whitespace) is captured into rest.
  command = strlower(command)
  if command == "" then
    --/bloot 
    --BLTabContainerFrame:Show();
  	sendMessage("GUI has not been implemented yet - sorry.");
  elseif command == "clear" or command == "c" then
    --/bloot clear
    actionClearList(rest)
  elseif command == "show" or command == "s" then
    --/bloot show
    actionBroadcastList("SELF");
  elseif command == "broadcast" or command == "b" then
    --/bloot broadcast
    actionBroadcastList("RAID");
  elseif command == "add" or command == "a" then
    --/bloot add <who> <item>
  	local lootData = {};
    lootData.who, lootData.item = rest:match("^(%S*)%s*(.-)$");
    lootData.id = generateLootId();
	  lootData.when = getServerDate(true); -- now!!!
    if isValidCharacter(lootData.who) and isValidItem(lootData.item) then
      actionAddLoot(lootData);
    else
      displayUsageAdd(true);
    end
  elseif command == "roll" or command =="r" then
    actionStartRolls();
  elseif command == "debug" then
    if strlower(rest) == "on" then
      BLootAddonSaved.Config.debugMode = 1;
      sendMessage("Debug Enabled")
    elseif strlower(rest) == "off" then
      BLootAddonSaved.Config.debugMode = 0;
      sendMessage("Debug Disabled")
    else
      sendMessage("Unknown argument to debug.")
    end
  elseif command == "debugnospam" then
    if strlower(rest) == "on" then
      BLootAddonSaved.Config.debugMode_NoSpam = 1;
      sendMessage("Debug No Spam Enabled")
    elseif strlower(rest) == "off" then
      BLootAddonSaved.Config.debugMode_NoSpam = 0;
      sendMessage("Debug No Spam Disabled")
    else
      sendMessage("Unknown argument to debugnospam.")
    end
  elseif command == "countdown" then
    -- need some validation on the value
    local countdown = tonumber(rest)
    if countdown == nil then
      sendMessage("Unknown argument to countdown '"..rest.."'.")
    else
      if countdown >= 10 and countdown <= 30 then
        BLootAddonSaved.Config.Countdown = countdown;
        sendMessage("Countdown set to "..countdown)
      else
        sendMessage("Countdown can only be set between 10 and 30 seconds long")
      end
    end
  else
  -- If not handled above, display some sort of help message
    displayUsageAll(true);
  end
end

function displayUsageAll(header)
  if header == true then
    sendMessage("Syntax:");
  end
  displayUsageClear(false);
  displayUsageAdd(false)
end
function displayUsageClear(header)
  if header == true then
    sendMessage("Syntax:");
  end
  sendMessage("/bloot clear");
  sendMessage("   Clears the current loot list");
end
function displayUsageAdd(header)
  if header == true then
    sendMessage("Syntax:");
  end
  sendMessage("/bloot add <who> <item>");
  sendMessage("   Adds an item to the list");
  sendMessage("     <who>  - Name of who gave the item");
  sendMessage("     <item> - Link to the item (shift click)");
end

---------
-- ACTIONS
function actionClearList()
  -- very simply just destroy the object for now...
  BLootAddonSaved.LootList = {};
  sendMessage("Loot ist cleared");
end
function actionBroadcastList(toWho)
  actionBroadcastListViaChat(toWho)
  actionBroadcastListViaAddon(toWho)
end
function actionBroadcastListViaChat(toWho)
  if BLootAddonSaved.LootList == nil then
    sendMessage("No loot in list", "SELF");
  else
    local numItems = #(BLootAddonSaved.LootList) 
    if numItems > 0 then
      sendMessage("We have got "..numItems.." bits of loot to give away.", toWho); 
      for index, lootData in ipairs(BLootAddonSaved.LootList) do
        sendMessage(lootData.item.." given by ".. lootData.who..".", toWho); 
      end
    else
      sendMessage("No loot in list", "SELF");
    end
  end
end
function actionBroadcastListViaAddon()
  --sendMessage("actionBroadcastListViaAddon - NYI");
end
function actionAddLoot(lootData)
  if BLootAddonSaved.LootList == nil then
    BLootAddonSaved.LootList = {};
  end
  tinsert(BLootAddonSaved.LootList, lootData);
  actionAnounceItem(lootData)
end
function actionAnounceItem(lootData)
  actionAnounceViaChat(lootData);
  actionAnounceViaAddon(lootData);
end
function actionAnounceViaChat(lootData)
  sendMessage(lootData.item.." given by ".. lootData.who..".", toWho);
end
function actionAnounceViaAddon(lootData)
  --sendMessage("actionAnounceViaAddon - NYI");
end
function actionStartRolls()
  sendMessage("actionStartRolls()", "DEBUG");
  if BLootAddonSaved.LootList == nil then
    sendMessage("No loot in list to roll for", "RAID");
  else
    local numItems = #(BLootAddonSaved.LootList) 
    if numItems > 0 then
      sendMessage("We have got "..numItems.." bits of loot to give away.", "RAID");
      if BLootAddonSaved.State == nill then
        BLootAddonSaved.State = {}
      elseif BLootAddonSaved.State.RollInProgress == true and not(moreThanAMinuteAgo(BLootAddonSaved.State.RollStarted) ) then
        -- we have a roll already in progress so shouldn't try to start another one.
        sendMessage("Rolls are already in progress.... or something has gone wrong.");
      else
        sendMessage("  actionStartRolls() - starting to give it all away", "DEBUG");
        BLootAddonSaved.State.LootToRollOn = 1;
        actionStartRollForItem();
      end
    else
      sendMessage("No loot in list to roll for", "RAID");
    end
  end
end

function actionStartRollForItem()
  sendMessage("actionStartRollForItem()", "DEBUG");
  local numItems = #(BLootAddonSaved.LootList)
  sendMessage("  actionStartRollForItem() numItems "..numItems..".", "DEBUG");
  if numItems > 0 then
    sendMessage("  actionStartRollForItem() BLootAddonSaved.State.LootToRollOn "..BLootAddonSaved.State.LootToRollOn..".", "DEBUG");
    if BLootAddonSaved.State.LootToRollOn <= numItems then
      BLootAddonSaved.State.RollInProgress = true;
      BLootAddonSaved.State.Countdown = BLootAddonSaved.Config.Countdown;
      BLootAddonSaved.State.Rolls = {}
      BLootAddonSaved.State.RollStarted = getServerDate(true)
      BLootAddonSaved.State.Winner = nil
      sendMessage("  actionStartRollForItem() finding item.", "DEBUG");
      BLootAddonSaved.State.RollingOn = BLootAddonSaved.LootList[BLootAddonSaved.State.LootToRollOn]
      sendMessage(BLootAddonSaved.State.RollingOn, "DEBUG");
      sendMessage("Starting rolls for "..BLootAddonSaved.State.RollingOn.item.." given by "..BLootAddonSaved.State.RollingOn.who..".", "RAID");
      C_Timer.After(1, actioRollForItemTick) 
    else
      sendMessage("No more items.", "RAID");
    end
  end
end

function actioRollForItemTick()
  sendMessage("actioRollForItemTick", "DEBUG");
  if BLootAddonSaved.State.RollInProgress then
    if BLootAddonSaved.State.Countdown == 0 then
      sendMessage("0 - Times up!", "RAID");
      actionRollForItemConfirmWinner()
    else
      if BLootAddonSaved.State.Countdown > 10 then
        if mod(BLootAddonSaved.State.Countdown, 5) == 0 then
          sendMessage(BLootAddonSaved.State.Countdown, "RAID");
        end
      else
        sendMessage(BLootAddonSaved.State.Countdown, "RAID");
      end
      BLootAddonSaved.State.Countdown = BLootAddonSaved.State.Countdown - 1;
      C_Timer.After(1, actioRollForItemTick)
    end
  end
end

function highestRoll(a, b) return a.roll > b.roll end

function actionRollForItemConfirmWinner()
  local numRolls = #(BLootAddonSaved.State.Rolls);
  if numRolls==0 then
    sendMessage("No one wanted "..BLootAddonSaved.State.RollingOn.item..", guess it goes into the bin!", "RAID");
  else
    sendMessage("We have "..numRolls.." people who have rolled on "..BLootAddonSaved.State.RollingOn.item..".", "RAID");
    sort(BLootAddonSaved.State.Rolls, highestRoll)
    BLootAddonSaved.State.Winner = BLootAddonSaved.State.Rolls[1];   
    actionAnnounceWinner();
  end
  BLootAddonSaved.State.LootToRollOn = BLootAddonSaved.State.LootToRollOn + 1;
  actionStartRollForItem();
end
function actionAnnounceWinner()
  actionAnnounceWinnerChat();
  actionAnnounceWinnerAddon();
end
function actionAnnounceWinnerChat()
  sendMessage("The winner of  "..BLootAddonSaved.State.RollingOn.item.." is "..BLootAddonSaved.State.Winner.name.." with a roll of "..BLootAddonSaved.State.Winner.roll, "RAID");
end
function actionAnnounceWinnerAddon()
  --sendMessage("actionAnnounceWinnerAddon - NYI");
end
function actionTradeLoot(lootData)
  sendMessage("actionTradeLoot - NYI");
end

---------
-- HANDLERS
function handleRoll(name, roll, minRoll, maxRoll)
  if BLootAddonSaved.State.RollInProgress then
    if BLootAddonSaved.State.Countdown > 0 then
  -- Should we really care?
      if minRoll == "1" and maxRoll == "100" then
        sendMessage(name.." rolled "..roll..".", "DEBUG")
        tinsert(BLootAddonSaved.State.Rolls, {name = name, roll = roll})
      else
        sendMessage(name.." is a cheat!!!!!"..minRoll.."/"..maxRoll)
      end
    end
  end
end

function handleRegisterInterest(id, name, item)

end
      


---------
-- UTILITY FUNCTIONS
function sendMessage(message, channel)
  if message == nil then
    message = "";
  end;
  if type(message) == "table" then
    local tableMessage = message
    message = "["
    for index,value in pairs(tableMessage) do 
      if type(value) == "table" then
        local tableMessage2 = value
        message = message.."["
        for index2,value2 in pairs(tableMessage2) do 
          if type(value2) == "table" then
            message = message..tostring(index2).." : [another table], "
          else
            message = message..tostring(index2).." : "..value2..", "
          end
        end
        message = message.."]"
      else
        message = message..tostring(index).." : "..value..", "
      end
    end
    message = message.."]"
  end
  if channel == nil then
    channel = "SELF";
  end;
  if BLootAddonSaved.Config.debugMode_NoSpam == 1 then
    DEFAULT_CHAT_FRAME:AddMessage("Broken's Loot ("..channel.."): "..message);
  else
    if channel=="RAID" then
      if GetNumGroupMembers() > 5 then
        SendChatMessage("Broken's Loot: "..message, "RAID");
      elseif GetNumGroupMembers() > 0 then
        SendChatMessage("Broken's Loot: "..message, "PARTY");
      else 
        DEFAULT_CHAT_FRAME:AddMessage("Broken's Loot: "..message);
      end
    elseif (channel=="GUILD") then
        SendChatMessage("Broken's Loot: "..message, "GUILD") 
    elseif (channel=="SELF") then
      DEFAULT_CHAT_FRAME:AddMessage("Broken's Loot: "..message);
    elseif (channel=="DEBUG") and (BLootAddonSaved.Config.debugMode == 1) then
      DEFAULT_CHAT_FRAME:AddMessage("Broken's Loot (DEBUG): "..message, 1, 1, 0);
    end
  end
end

function isValidCharacter(characterName)
  local i = getCharacterGroupIndex(characterName);
  sendMessage("isValidCharacter('"..characterName.."') is group at: "..i..".", "DEBUG");
  if i > 0 then
  	return true
  else
    sendMessage("isValidCharacter('"..characterName.."') is not in the group", "DEBUG");
    local numGroupMembers = GetNumGroupMembers()
    if numGroupMembers == 0 then
      local name = GetUnitName("player", true)
      if strlower(name) == strlower(characterName) then
        sendMessage("isValidCharacter('"..characterName.."') is the current character", "DEBUG");
        return true
      end
    end
  end
  return false
end

function getCharacterGroupIndex(characterName)
  sendMessage("getCharacterGroupIndex('"..characterName.."')", "DEBUG");
  local numGroupMembers = GetNumGroupMembers()
  for ci = 1, numGroupMembers do
    if strlower(GetRaidRosterInfo(ci)) == strlower(characterName) then
      sendMessage("getCharacterGroupIndex('"..characterName.."') = "..ci, "DEBUG");
      return ci;
    end
  end
  sendMessage("getCharacterGroupIndex('"..characterName.."') = not in group!", "DEBUG");
  return 0;
end

function isValidItem(itemLink)
  sendMessage("isValidItem - NYI");
  debugItem(itemLink)
  return true;
end

function debugItem(itemLink)
  sendMessage("debugItem("..itemLink..")", "DEBUG");
  local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemLink) 
  sendMessage("  itemType:"..itemType, "DEBUG");
  sendMessage("  itemSubType:"..itemSubType, "DEBUG");
  sendMessage("  itemEquipLoc"..itemEquipLoc, "DEBUG");
  getItemStats(itemLink);
end;

function getItemStats(itemLink)
  local stats = {};
  GetItemStats(itemLink, stats);
  sendMessage("getItemStats("..itemLink..")", "DEBUG")
  for key, value in pairs(stats) do 
    sendMessage("  "..key.." - " ..value, "DEBUG")
  end
  local agility, intellect, strength, stamina;
  agility = (stats["ITEM_MOD_AGILITY_SHORT"] or 0);
  intellect = (stats["ITEM_MOD_INTELLECT_SHORT"] or 0);
  strength = (stats["ITEM_MOD_STRENGTH_SHORT"] or 0);
  stamina = (stats["ITEM_MOD_STAMINA_SHORT"] or 0);
  sendMessage("  agi "..agility.." int "..intellect.." str "..strength.." stam "..stamina, "DEBUG");
  return agility, intellect, strength, stamina;
end

function generateLootId()
  BLootAddonSaved.lootId = BLootAddonSaved.lootId + 1;
  sendMessage("generateLootId() = "..BLootAddonSaved.lootId, "DEBUG")
  return BLootAddonSaved.lootId;
end

function getServerDate(andTime)
  local weekday, month, day, year = CalendarGetDate();
  if month < 10 then month = "0"..month; end;
  if day < 10 then day = "0"..day; end;
  if andTime then
    local hours,minutes = GetGameTime();
    if hours < 10 then hours = "0"..hours; end;
    if minutes < 10 then minutes = "0"..minutes; end;
    local currentDateTime = day.."/"..month.."/"..year.." "..hours..":"..minutes..":00";
    return currentDateTime;
  else
    local currentDate = day.."/"..month.."/"..year;
    return currentDate;
  end;
end

function moreThanAMinuteAgo(when)
-- NYI
  return true;
end

---------
-- STARTUP
SLASH_BROKENSLOOT1 = '/bloot';
SlashCmdList["BROKENSLOOT"] = handler;

function startup()
  localized = localizeStrings();
  if BLootAddonSaved == nil then
    BLootAddonSaved = {}
    BLootAddonSaved.lootId = 0;
  end
  if BLootAddonSaved.Config == nill then
    BLootAddonSaved.Config = {}
  end
  if BLootAddonSaved.Config.debugMode == nil then 
    BLootAddonSaved.Config.debugMode = 0; 
  elseif BLootAddonSaved.Config.debugMode == 1 then 
    sendMessage("Debug: on", "SELF") 
  end;
  if BLootAddonSaved.Config.debugMode_NoSpam == nil then 
    BLootAddonSaved.Config.debugMode_NoSpam = 0; 
  elseif BLootAddonSaved.Config.debugMode_NoSpam == 1 then 
    sendMessage("Debug No Spam: on", "SELF") 
  end;
  if BLootAddonSaved.Config.Countdown == nil then 
    BLootAddonSaved.Config.Countdown = 30; 
  elseif BLootAddonSaved.Config.debugMode == 1 then 
    sendMessage("Countdown: "..BLootAddonSaved.Config.Countdown, "SELF") 
  end;
  sendMessage("Addon loaded...");
end


---------
-- ADDON VARIABLES
BrokensLoot = {}; -- holder for the panels
localized = {};

---------
-- EVENTS
local frame, events = CreateFrame("Frame"), {};

function events:CHAT_MSG_SYSTEM (...)
  local arg1 = select(1,...)
  if arg1 then
    --sendMessage("localized.rollResult: " .. localized.rollResult)
    local name, roll, minRoll, maxRoll = arg1:match(localized.rollResult)
    if name then
      -- we have found a roll - I think :)
      handleRoll(name, roll, minRoll, maxRoll);
    end
  end
end

function events:VARIABLES_LOADED (...)
  startup();
end

frame:SetScript("OnEvent", function(self, event, ...)
  events[event](self, ...); -- call one of the functions above
end);

for k, v in pairs(events) do
  frame:RegisterEvent(k); -- Register all events for which handlers have been defined
end

---------
-- LOCALISATION
function localizeStrings()
  local loc = {};
  loc.rollResult = RANDOM_ROLL_RESULT;
  --sendMessage("START:" .. loc.rollResult)
  --sendMessage("WANT :" .. "^(.+) rolls (%d+) %((%d+)%-(%d+)%)$");
  -- ("^(.+) rolls (%d+) %((%d+)%-(%d+)%)$")
  -- ("%s rolls %d (%d-%d)")
  loc.rollResult = gsub(loc.rollResult, "%%s", "^(.+)", 1)
  --sendMessage("GOT1 :" .. loc.rollResult)
  loc.rollResult = gsub(loc.rollResult, "%%d", "(%%d+)", 1)
  --sendMessage("GOT2 :" .. loc.rollResult)
  loc.rollResult = gsub(loc.rollResult, "%(%%d%-%%d%)", "%%((%%d+)%%-(%%d+)%%)$", 1)
  --sendMessage("GOT3 :" .. loc.rollResult)
  --sendMessage("WANT :" .. "^(.+) rolls (%d+) %((%d+)%-(%d+)%)$");
  return loc;
end
